local ADDON, Package = ...
local Color = Package.Color

local function chat_sticky_raid_warning()
  ChatTypeInfo.RAID_WARNING.sticky = 1
end

local function chat_unclmap_frame()
  for at = 1, 10 do
    local frame = _G["ChatFrame" .. at]

    if at ~= 2 and frame then
      frame:SetClampRectInsets(0, 0, 0, 0)
    end
  end

  local function FloatingChatFrame_UpdateBackgroundAnchors(self)
    self:SetClampRectInsets(0, 0, 0, 0)
  end

  hooksecurefunc("FloatingChatFrame_UpdateBackgroundAnchors", FloatingChatFrame_UpdateBackgroundAnchors)
end

local function chat_max_history()
  for at = 1, 10 do
    local frame = _G["ChatFrame" .. at]

    if at ~= 2 and frame then
      frame:SetMaxLines(4096)
    end
  end
end

local function chat_arrow_keys()
  for at = 1, 10 do
    local frame = _G["ChatFrame" .. at]

    if at ~= 2 and frame and frame.editBox then
      frame.editBox:SetAltArrowKeyMode(false)
    end
  end
end

local function chat_message_logger()
  local function darken_message_content(hex_string)
    local r, g, b = Color.to_rgb_from_hex(tonumber(hex_string, 16))

    return Color.to_hex_markup_from_rgb(Color.darken(r, g, b, 20))
  end

  local function darken_message(message, r, g, b, ...)
    if message then
      r, g, b = Color.darken(r or 1, g or 1, b or 1, 20)
      message = message:gsub("|cff(%x%x%x%x%x%x)", darken_message_content)
    end

    return message, r, g, b, ...
  end

  for frame_name, messages in pairs(MedusaPreludeStoragePerCharacter.chat_log) do
    local frame = _G[frame_name]

    for at = #messages, 1, -1 do
      frame:AddMessage(darken_message(unpack(messages[at])))
    end
  end

  local function replace_session_id_with_battle_tag(message)
    if message and message:find("|HBNplayer:.+|h") then
      local account_id = message:match("|HBNplayer:[^:]+:([0-9]+).+|h") or 0
      local _, _, battle_tag = BNGetFriendInfoByID(account_id)

      if battle_tag then
        message = message:gsub("|K[^|]+|k", battle_tag)
      end
    end

    return message
  end

  local function AddMessage(self, message, ...)
    local frame_name = self:GetName()
    local messages = MedusaPreludeStoragePerCharacter.chat_log[frame_name]

    table.insert(messages, 1, {replace_session_id_with_battle_tag(message), ...})

    if #messages > 64 then
      table.remove(messages)
    end
  end

  for at = 1, 10 do
    local frame = _G["ChatFrame" .. at]

    if at ~= 2 and frame then
      hooksecurefunc(frame, "AddMessage", AddMessage)
    end
  end
end

local function chat_copy_url_to_clipboard()
  StaticPopupDialogs[ADDON .. "Clipboard"] = {
    text = "",
    button1 = CLOSE,
    timeout = 0,
    whileDead = true,
    hasEditBox = true,
    hideOnEscape = true,
    OnShow = function (self)
      self.editBox:SetText(self.data)
      self.editBox:SetCursorPosition(0)
      self.editBox:HighlightText()
      self.editBox:SetScript("OnMouseUp", function (self) self:HighlightText() end)
    end,
    OnHide = function (self)
      self.editBox:SetScript("OnMouseUp", nil)
    end,
    EditBoxOnEscapePressed = function (self)
      self:GetParent():Hide()

      ClearCursor()
    end,
    EditBoxOnTextChanged = function (self)
      self:SetText(self:GetParent().data)
      self:SetCursorPosition(0)
      self:HighlightText()
    end
  }

  local function ChatFrame_OnHyperlinkShow(self, link, text, button)
    if link:sub(1, 4) == "item" and text:find("://") then
      StaticPopup_Show(ADDON .. "Clipboard", nil, nil, text:match("%[(.+)%]"))
    end
  end

  hooksecurefunc("ChatFrame_OnHyperlinkShow", ChatFrame_OnHyperlinkShow)

  local function MessageEventFilter(self, event, message, ...)
    if message and message:find("://") then
      local pre, url, post = message:match("^(.*)(https?://[^ ]+)(.*)$")

      if pre and url and post then
        message = string.format("%s|H%s:|h[%s]|h%s", pre, "item", url, post)
      end
    end

    return false, message, ...
  end

  local EVENTS = {
    "AFK",
    "BN_WHISPER",
    "BN_WHISPER_INFORM",
    "BN_WHISPER_PLAYER_OFFLINE",
    "CHANNEL",
    "CHANNEL_NOTICE",
    "CHANNEL_NOTICE_USER",
    "DND",
    "EMOTE",
    "GUILD",
    "INSTANCE_CHAT",
    "INSTANCE_CHAT_LEADER",
    "OFFICER",
    "PARTY",
    "PARTY_LEADER",
    "RAID",
    "RAID_LEADER",
    "RAID_WARNING",
    "REPLY",
    "SAY",
    "SYSTEM",
    "TEXT_EMOTE",
    "WHISPER",
    "WHISPER_INFORM",
    "YELL",
  }

  for _, event in pairs(EVENTS) do
    ChatFrame_AddMessageEventFilter("CHAT_MSG_" .. event, MessageEventFilter)
  end
end

local function chat_copy_text_to_clipboard()
  local function OnClick(self)
    local is_flagged = self:GetChecked()

    self:SetButtonState("PUSHED", is_flagged)

    for at = 1, 10 do
      local frame = _G["ChatFrame" .. at]
      frame:EnableMouse(is_flagged)
      frame:SetTextCopyable(is_flagged)
    end
  end

  local function OnEnter(self)
    self:SetAlpha(1)
  end

  local function OnLeave(self)
    self:SetAlpha(0.5)
  end

  local frame = CreateFrame("CheckButton", ADDON .. "ChatClipboardButton", ChatFrame1)
  frame:SetAlpha(0.5)
  frame:SetSize(16, 16)
  frame:SetPoint("TOPRIGHT")
  frame:SetNormalTexture([[Interface\AddOns\MedusaPrelude\media\clipboard-solid.tga]])
  frame:SetPushedTexture([[Interface\AddOns\MedusaPrelude\media\cut-solid.tga]])
  frame:SetScript("OnClick", OnClick)
  frame:SetScript("OnEnter", OnEnter)
  frame:SetScript("OnLeave", OnLeave)

  local background = frame:CreateTexture(nil, "BACKGROUND")
  background:SetAllPoints(frame)
  background:SetTexture([[Interface\Buttons\WHITE8X8]])
  background:SetVertexColor(0.5, 0.5, 0.5, 0.5)
end

Package.chat_sticky_raid_warning = chat_sticky_raid_warning
Package.chat_unclmap_frame = chat_unclmap_frame
Package.chat_max_history = chat_max_history
Package.chat_arrow_keys = chat_arrow_keys
Package.chat_message_logger = chat_message_logger
Package.chat_copy_url_to_clipboard = chat_copy_url_to_clipboard
Package.chat_copy_text_to_clipboard = chat_copy_text_to_clipboard
